--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity COLSEL is
	port (
		MDSEL		: in  std_logic;
		LOCK_PLL	: in  std_logic;
		CLK50M		: in  std_logic;
		RSTN		: in  std_logic;
		COLORMD		: out std_logic_vector(2 downto 0);
		SEL640		: out std_logic
	);
end COLSEL;

architecture RTL of COLSEL is
	signal mdsel_f1		: std_logic;
	signal mdsel_f2		: std_logic;
	signal cnt10ms		: std_logic_vector(18 downto 0);
	signal cnton		: std_logic_vector(3 downto 0);
	signal cntonlong	: std_logic_vector(6 downto 0);
	signal cntoff		: std_logic_vector(3 downto 0);
	signal cntofflong	: std_logic_vector(4 downto 0);
	signal initrd_f1	: std_logic;
	signal initrd_f2	: std_logic;
	signal initrd_f3	: std_logic;
	signal keyin		: std_logic;
	signal keyin_f1		: std_logic;
	signal keylongin	: std_logic;
	signal keylongin_f1	: std_logic;
	signal colormd_i	: std_logic_vector(2 downto 0);
	signal sel640_i		: std_logic;


begin

-- input latch
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			mdsel_f1 <= '0';
			mdsel_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			mdsel_f1 <= MDSEL;
			mdsel_f2 <= mdsel_f1;
		end if;
	end process;


-- 10ms counter
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cnt10ms <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (cnt10ms = 499999) then
				cnt10ms <= (others => '0');
			else
				cnt10ms <= cnt10ms + 1;
			end if;
			cnt10ms <= cnt10ms + 1;
		end if;
	end process;

-- chattering cancel
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			initrd_f1 <= '1';
			initrd_f2 <= '1';
			initrd_f3 <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (LOCK_PLL = '1') then
				initrd_f1 <= '0';
				initrd_f2 <= initrd_f1;
				initrd_f3 <= initrd_f2;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cnton      <= (others => '0');
			cntonlong  <= (others => '0');
			cntoff     <= (others => '0');
			cntofflong <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (initrd_f3 = '1') then
				cnton     <= (others => '0');
				cntonlong <= (others => '0');
				cntoff    <= (others => '0');
				cntofflong <= (others => '0');
			elsif (cnt10ms = 0) then
				if (mdsel_f2 = '1') then
					if (cnton      /= "1111") then
						cnton      <= cnton + 1;
					end if;
					if (cntonlong  /= "1111111") then
						cntonlong  <= cntonlong + 1;
					end if;
					cntoff     <= (others => '0');
					cntofflong <= (others => '0');
				else
					if (cntoff     /= "1111") then
						cntoff     <= cntoff + 1;
					end if;
					if (cntofflong /= "11111") then
						cntofflong <= cntofflong + 1;
					end if;
					cnton      <= (others => '0');
					cntonlong  <= (others => '0');
				end if;
			end if;
		end if;
	end process;

-- mode select input
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			keyin    <= '0';
			keyin_f1 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (initrd_f3 = '1') then
				keyin <= '0';
			elsif (cnton  = "1111") then
				keyin <= '1';
			elsif (cntoff = "1111") then
				keyin <= '0';
			end if;
			keyin_f1 <= keyin;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			keylongin    <= '1';
			keylongin_f1 <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (initrd_f3 = '1') then
				keylongin <= '1';
			elsif (cntonlong  = "1111111") then
				keylongin <= '1';
			elsif (cntofflong = "11111") then
				keylongin <= '0';
			end if;
			keylongin_f1 <= keylongin;
		end if;
	end process;

-- mode select
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			colormd_i <= (others => '0');
			sel640_i  <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (initrd_f3 = '1') then
				colormd_i <= (others => '0');
				sel640_i  <= '1';
			elsif (keylongin = '1' and keylongin_f1 = '0') then
				sel640_i <= not sel640_i;
			elsif (keylongin = '0') then
				if (keyin = '0' and keyin_f1 = '1') then
					if (colormd_i = "000") then
						colormd_i <= "100";
					else
						colormd_i <= colormd_i + 1;
					end if;
				end if;
			end if;
		end if;
	end process;

	COLORMD <= colormd_i;
	SEL640  <= sel640_i;

end RTL;
