--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity P6SYNC is
	port (
		P6_Y_SYNC	: in  std_logic;					-- Y SYNC
		P6_YBR		: in  std_logic_vector(5 downto 0);	-- Y VH/VL B VH/VL R VH/VL
		P6_CLK		: in  std_logic;
		CLK			: in  std_logic;
		RSTN		: in  std_logic;
		CLKENB		: out std_logic;
		DT_HSYNCN	: out std_logic;
		DT_VSYNCN	: out std_logic;
		DT_RGBH		: out std_logic_vector(3 downto 0);
		LOCK1		: out std_logic;
		LOCK2		: out std_logic;
		LOCK3		: out std_logic;
		LOCK4		: out std_logic;
		LOCK5		: out std_logic
	);
end P6SYNC;

architecture RTL of P6SYNC is

	signal p6_clk_f		: std_logic_vector(41 downto 0);
	signal p6_hsyncn_f1	: std_logic;
	signal p6_hsyncn_f2	: std_logic;
	signal p6_hsyncn_f3	: std_logic;
	signal p6_hsyncn_f4	: std_logic;
	signal p6_hsyncn_f5	: std_logic;
	signal p6_rgbh_f1	: std_logic_vector(5 downto 0);
	signal p6_rgbh_f2	: std_logic_vector(5 downto 0);
	signal p6_rgbh_f3	: std_logic_vector(5 downto 0);
	signal p6_rgbh_f4	: std_logic_vector(5 downto 0);
	signal p6_rgbh_f5	: std_logic_vector(5 downto 0);

	signal det_yh_dtf	: std_logic_vector(30 downto 0);
	signal det_yl_dtf	: std_logic_vector(30 downto 0);

	signal hcnt			: std_logic_vector(7 downto 0);
	signal vcnt			: std_logic_vector(8 downto 0);

	signal clkr			: std_logic;
	signal clkf			: std_logic;
	signal clkr_f		: std_logic_vector(69 downto 0);
	signal clkf_f		: std_logic_vector(69 downto 0);
	signal clkinv		: std_logic;
	signal clkinvcnt	: std_logic_vector(1 downto 0);

	signal sync_tmg1	: std_logic;
	signal sync_tmg2	: std_logic;
	signal sync_tmg3	: std_logic;
	signal sync_tmg4	: std_logic;
	signal sync_tmg5	: std_logic;
	signal sync_inv1	: std_logic;
	signal sync_inv2	: std_logic;
	signal sync_inv3	: std_logic;
	signal sync_inv4	: std_logic;
	signal sync_inv5	: std_logic;
	signal sync_ng1		: std_logic;
	signal sync_ng2		: std_logic;
	signal sync_ng3		: std_logic;
	signal sync_ng4		: std_logic;
	signal sync_ng5		: std_logic;
	signal sync_ng		: std_logic;
	signal sync_tmg		: std_logic;
	signal sync_tmg_f1	: std_logic;

	signal yh_tmg1		: std_logic;
	signal yh_tmg2		: std_logic;
	signal yh_tmg3		: std_logic;
	signal yh_tmg4		: std_logic;
	signal yh_tmg5		: std_logic;
	signal yh_ng1		: std_logic;
	signal yh_ng2		: std_logic;
	signal yh_ng3		: std_logic;
	signal yh_ng4		: std_logic;
	signal yh_ng5		: std_logic;
	signal yh_ng		: std_logic;
	signal yh_tmg		: std_logic;
	signal yh_tmg_f1	: std_logic;
	signal yl_tmg1		: std_logic;
	signal yl_tmg2		: std_logic;
	signal yl_tmg3		: std_logic;
	signal yl_tmg4		: std_logic;
	signal yl_tmg5		: std_logic;
	signal yl_ng1		: std_logic;
	signal yl_ng2		: std_logic;
	signal yl_ng3		: std_logic;
	signal yl_ng4		: std_logic;
	signal yl_ng5		: std_logic;
	signal yl_ng		: std_logic;
	signal yl_tmg		: std_logic;
	signal yl_tmg_f1	: std_logic;
	signal bh_tmg1		: std_logic;
	signal bh_tmg2		: std_logic;
	signal bh_tmg3		: std_logic;
	signal bh_tmg4		: std_logic;
	signal bh_tmg5		: std_logic;
	signal bh_ng1		: std_logic;
	signal bh_ng2		: std_logic;
	signal bh_ng3		: std_logic;
	signal bh_ng4		: std_logic;
	signal bh_ng5		: std_logic;
	signal bh_ng		: std_logic;
	signal bh_tmg		: std_logic;
	signal bh_tmg_f1	: std_logic;
	signal bl_tmg1		: std_logic;
	signal bl_tmg2		: std_logic;
	signal bl_tmg3		: std_logic;
	signal bl_tmg4		: std_logic;
	signal bl_tmg5		: std_logic;
	signal bl_ng1		: std_logic;
	signal bl_ng2		: std_logic;
	signal bl_ng3		: std_logic;
	signal bl_ng4		: std_logic;
	signal bl_ng5		: std_logic;
	signal bl_ng		: std_logic;
	signal bl_tmg		: std_logic;
	signal bl_tmg_f1	: std_logic;
	signal rh_tmg1		: std_logic;
	signal rh_tmg2		: std_logic;
	signal rh_tmg3		: std_logic;
	signal rh_tmg4		: std_logic;
	signal rh_tmg5		: std_logic;
	signal rh_ng1		: std_logic;
	signal rh_ng2		: std_logic;
	signal rh_ng3		: std_logic;
	signal rh_ng4		: std_logic;
	signal rh_ng5		: std_logic;
	signal rh_ng		: std_logic;
	signal rh_tmg		: std_logic;
	signal rh_tmg_f1	: std_logic;
	signal rl_tmg1		: std_logic;
	signal rl_tmg2		: std_logic;
	signal rl_tmg3		: std_logic;
	signal rl_tmg4		: std_logic;
	signal rl_tmg5		: std_logic;
	signal rl_ng1		: std_logic;
	signal rl_ng2		: std_logic;
	signal rl_ng3		: std_logic;
	signal rl_ng4		: std_logic;
	signal rl_ng5		: std_logic;
	signal rl_ng		: std_logic;
	signal rl_tmg		: std_logic;
	signal rl_tmg_f1	: std_logic;

	signal clkenb_f1	: std_logic;

	signal det_ys		: std_logic;
	signal det_ys_flag	: std_logic;
	signal dt_val		: std_logic;

	signal det_yh		: std_logic;
	signal det_yl		: std_logic;
	signal yhdt_val		: std_logic;
	signal yldt_val		: std_logic;
	signal det_bh		: std_logic;
	signal det_bl		: std_logic;
	signal det_rh		: std_logic;
	signal det_rl		: std_logic;

	signal sync_det_cnt	: std_logic_vector(7 downto 0);
	signal cnt_sync		: std_logic_vector(39 downto 0);
	signal cnt_sync_lt	: std_logic_vector(39 downto 0);

	signal cnt_yh		: std_logic_vector(39 downto 0);
	signal cnt_yh_lt	: std_logic_vector(39 downto 0);
	signal cnt_yl		: std_logic_vector(39 downto 0);
	signal cnt_yl_lt	: std_logic_vector(39 downto 0);

	signal cnt_bh		: std_logic_vector(39 downto 0);
	signal cnt_bh_lt	: std_logic_vector(39 downto 0);
	signal cnt_bl		: std_logic_vector(39 downto 0);
	signal cnt_bl_lt	: std_logic_vector(39 downto 0);

	signal cnt_rh		: std_logic_vector(39 downto 0);
	signal cnt_rh_lt	: std_logic_vector(39 downto 0);
	signal cnt_rl		: std_logic_vector(39 downto 0);
	signal cnt_rl_lt	: std_logic_vector(39 downto 0);

	signal monlat_f1	: std_logic;
	signal monlat_f2	: std_logic;
	signal monlat_f3	: std_logic;
	signal mondt0_i		: std_logic_vector(39 downto 0);
	signal mondt1_i		: std_logic_vector(39 downto 0);
	signal mondt2_i		: std_logic_vector(39 downto 0);
	signal mondt3_i		: std_logic_vector(39 downto 0);
	signal mondt4_i		: std_logic_vector(39 downto 0);
	signal mondt5_i		: std_logic_vector(39 downto 0);
	signal mondt6_i		: std_logic_vector(39 downto 0);

	signal dt_hsyncn_f1	: std_logic;
	signal dt_hsyncn_f2	: std_logic;
	signal dt_hsyncn_f3	: std_logic;
	signal dt_hsyncn_f4	: std_logic;
	signal dt_hsyncn_f5	: std_logic;
	signal dt_hsyncn_f6	: std_logic;
	signal dt_vsyncn_i	: std_logic;
	signal dt_rgbh_f1	: std_logic_vector(5 downto 0);
	signal dt_rgbh_f2	: std_logic_vector(5 downto 0);
	signal dt_rgbh_f3	: std_logic_vector(5 downto 0);
	signal dt_rgbh_f4	: std_logic_vector(5 downto 0);
	signal dt_rgbh_f5	: std_logic_vector(5 downto 0);
	signal dt_rgbh_f6	: std_logic_vector(5 downto 0);

	signal clkdet_i		: std_logic;
	signal dt_lock2		: std_logic;
	signal dt_lock3		: std_logic;
	signal dt_lock4		: std_logic;
	signal dt_lock41	: std_logic;
	signal dt_lock42	: std_logic;
	signal dt_lock5		: std_logic;
	signal dt_lock51	: std_logic;
	signal dt_lock52	: std_logic;
	signal dt_lock53	: std_logic;
	signal dt_lock54	: std_logic;

begin

-- input latch
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			p6_clk_f     <= (others => '0');
			p6_hsyncn_f1 <= '1';
			p6_hsyncn_f2 <= '1';
			p6_hsyncn_f3 <= '1';
			p6_hsyncn_f4 <= '1';
			p6_hsyncn_f5 <= '1';
			p6_rgbh_f1   <= "000000";
			p6_rgbh_f2   <= "000000";
			p6_rgbh_f3   <= "000000";
			p6_rgbh_f4   <= "000000";
			p6_rgbh_f5   <= "000000";
		elsif (CLK'event and CLK = '1') then
			p6_clk_f(0)   <= P6_CLK;
			p6_clk_f(1)   <= p6_clk_f(0) xor clkinv;
			p6_clk_f(41 downto 2) <= p6_clk_f(40 downto 1);
			p6_hsyncn_f1 <= P6_Y_SYNC;
			p6_hsyncn_f2 <= p6_hsyncn_f1;
			p6_hsyncn_f3 <= p6_hsyncn_f2;
			p6_hsyncn_f4 <= p6_hsyncn_f3;
			p6_hsyncn_f5 <= p6_hsyncn_f4;
			p6_rgbh_f1   <= P6_YBR;
			p6_rgbh_f2   <= p6_rgbh_f1;
			p6_rgbh_f3   <= p6_rgbh_f2;
			p6_rgbh_f4   <= p6_rgbh_f3;
			p6_rgbh_f5   <= p6_rgbh_f4;
		end if;
	end process;


-- CLK generate
	clkr <= '1' when (p6_clk_f(7 downto 4) = "0001") else '0';
	clkf <= '1' when (p6_clk_f(7 downto 4) = "1110") else '0';

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			clkr_f <= (others => '0');
			clkf_f <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			clkr_f <= clkr_f(68 downto 0) & clkr;
			clkf_f <= clkf_f(68 downto 0) & clkf;
		end if;
	end process;

-- clock detect
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			clkdet_i <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkr_f(1) = '1') then
				clkdet_i <= '1';
			elsif (	p6_clk_f(41 downto 2) = X"0000000000") then
				clkdet_i <= '0';
			elsif (	p6_clk_f(41 downto 2) = X"FFFFFFFFFF") then
				clkdet_i <= '0';
			end if;
		end if;
	end process;

-- sync edge detect
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			det_ys <= '0';
		elsif (CLK'event and CLK = '1') then
			if (p6_hsyncn_f2='1' and p6_hsyncn_f3='0' and p6_hsyncn_f4='0' and p6_hsyncn_f5='0') then
				det_ys <= '1';
			else
				det_ys <= '0';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			det_ys_flag <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys = '1') then
				det_ys_flag <= '1';
			elsif (clkr_f(1) = '1') then
				det_ys_flag <= '0';
			end if;
		end if;
	end process;

-- horizontal counter
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			hcnt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				hcnt <= (others => '0');
			elsif (clkr_f(1) = '1') then
				if (det_ys_flag = '1') then
					hcnt <= (others => '0');
				elsif (hcnt /= X"FF") then
					hcnt <= hcnt + 1;
				end if;
			end if;
		end if;
	end process;

-- vartical counter
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			vcnt <= (others => '1');
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				vcnt <= (others => '1');
			elsif (hcnt = 127 and clkr_f(1) = '1') then
				if (dt_hsyncn_f5 = '0' and vcnt > 2) then
					vcnt <= (others => '0');
				else
					vcnt <= vcnt + 1;
				end if;
			end if;
		end if;
	end process;

-- vsync generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_vsyncn_i <= '1';
		elsif (CLK'event and CLK = '1') then
			if (hcnt = 127) then
				dt_vsyncn_i <= dt_hsyncn_f5;
			end if;
		end if;
	end process;

-- data valid generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_val <= '0';
		elsif (CLK'event and CLK = '1') then
			if (hcnt >= 40 and hcnt < 215) then
				dt_val <= '1';
			else
				dt_val <= '0';
			end if;
		end if;
	end process;

-- sync pulse detect generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock2 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock2 <= '0';
			elsif (vcnt = 257) then
				dt_lock2 <= '1';
			end if;
		end if;
	end process;


-- data edge detect
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			det_yh_dtf <= (others => '0');
			det_yl_dtf <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (
				p6_rgbh_f2(5)='1' and
				p6_rgbh_f3(5)='0' and
				p6_rgbh_f4(5)='0' and
				p6_rgbh_f5(5)='0'
			) then
				det_yh_dtf(0) <= '1';
			elsif (
				p6_rgbh_f2(5)='0' and
				p6_rgbh_f3(5)='1' and
				p6_rgbh_f4(5)='1' and
				p6_rgbh_f5(5)='1'
			) then
				det_yh_dtf(0) <= '1';
			else
				det_yh_dtf(0) <= '0';
			end if;

			if (
				p6_rgbh_f2(4)='1' and
				p6_rgbh_f3(4)='0' and
				p6_rgbh_f4(4)='0' and
				p6_rgbh_f5(4)='0'
			) then
				det_yl_dtf(0) <= '1';
			elsif (
				p6_rgbh_f2(4)='0' and
				p6_rgbh_f3(4)='1' and
				p6_rgbh_f4(4)='1' and
				p6_rgbh_f5(4)='1'
			) then
				det_yl_dtf(0) <= '1';
			else
				det_yl_dtf(0) <= '0';
			end if;

			det_yh_dtf(30 downto 1) <= det_yh_dtf(29 downto 0);
			det_yl_dtf(30 downto 1) <= det_yl_dtf(29 downto 0);
		end if;
	end process;

	det_yh <=
	'0' when (dt_val = '0') else
	'0' when (yhdt_val = '0') else
	'1' when (det_yh_dtf(30) = '1') else
	'0';

	det_yl <=
	'0' when (dt_val = '0') else
	'0' when (yldt_val = '0') else
	'1' when (det_yl_dtf(30) = '1') else
	'0';

	det_bh <=
	'0' when (dt_val = '0') else
	'1' when (p6_rgbh_f2(3)='1' and p6_rgbh_f3(3)='0' and p6_rgbh_f4(3)='0' and p6_rgbh_f5(3)='0') else
	'1' when (p6_rgbh_f2(3)='0' and p6_rgbh_f3(3)='1' and p6_rgbh_f4(3)='1' and p6_rgbh_f5(3)='1') else
	'0';

	det_bl <=
	'0' when (dt_val = '0') else
	'1' when (p6_rgbh_f2(2)='1' and p6_rgbh_f3(2)='0' and p6_rgbh_f4(2)='0' and p6_rgbh_f5(2)='0') else
	'1' when (p6_rgbh_f2(2)='0' and p6_rgbh_f3(2)='1' and p6_rgbh_f4(2)='1' and p6_rgbh_f5(2)='1') else
	'0';

	det_rh <=
	'0' when (dt_val = '0') else
	'1' when (p6_rgbh_f2(1)='1' and p6_rgbh_f3(1)='0' and p6_rgbh_f4(1)='0' and p6_rgbh_f5(1)='0') else
	'1' when (p6_rgbh_f2(1)='0' and p6_rgbh_f3(1)='1' and p6_rgbh_f4(1)='1' and p6_rgbh_f5(1)='1') else
	'0';

	det_rl <=
	'0' when (dt_val = '0') else
	'1' when (p6_rgbh_f2(0)='1' and p6_rgbh_f3(0)='0' and p6_rgbh_f4(0)='0' and p6_rgbh_f5(0)='0') else
	'1' when (p6_rgbh_f2(0)='0' and p6_rgbh_f3(0)='1' and p6_rgbh_f4(0)='1' and p6_rgbh_f5(0)='1') else
	'0';


-- ͂f[^F279.4nsi3.579545MHzj  iBH/BL/RH/RLj
-- ͂f[^F139.7nsi3.579545MHzx2jiYS/YH/YLj
--
-- TvONbNF9.231nsi108.33MHzj
-- TvONbNF10.00nsi100.00MHzj
--
-- f[^TvOӏF30iBHA108.33MHzj
-- f[^TvOӏF28iBHA100.00MHzj

--
-- SYNC o͗オ݂̂Ď
-- P6_CLK ̗オ肩̂ǂ炩ɓĂ
-- P6̓dɌ܂̂ŁAoĂ
-- i̋Px^FMオ肩ɓĂ邽߁j
--
-- SYNC ̗́Aオ肩甼NbNĂ邪A
-- Ď荞łCɂȂigȂ߁j

--        +----+    +----+    +----+    +----+
-- CLK    |    |    |    |    |    |    |    |
--      --+    +----+    +----+    +----+    +----
--
--             +----------    ----------+
-- SYNC        |                        |
--      -------+           ~~           +---------
--
--      -------+---------+---------+---------+----
-- B/R         |         |         |         |
--      -------+---------+---------+---------+----
--
--      --+----+----+----+----+----+----+----+----
--  Y     |    |    |    |    |    |    |    |
--      --+----+----+----+----+----+----+----+----
--
-- CLK14MHẑŜ߁Aオ^肪
-- tɂȂ\
--


-- sync edge count
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sync_det_cnt <= X"00";
		elsif (CLK'event and CLK = '1') then
			if (det_ys = '1') then
				sync_det_cnt <= sync_det_cnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cnt_sync    <= (others => '0');
			cnt_sync_lt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (det_ys = '1') then
				if (sync_det_cnt = X"FF") then
					cnt_sync    <= (others => '0');
					cnt_sync_lt <= cnt_sync;
				else
					for i in 0 to 39 loop
						if (clkr_f(i+4) = '1') then
							cnt_sync(i) <= '1';
						end if;
					end loop;
				end if;
			end if;
		end if;
	end process;

-- sync latch timing generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sync_tmg1 <= '0';
			sync_tmg2 <= '0';
			sync_tmg3 <= '0';
			sync_tmg4 <= '0';
			sync_tmg5 <= '0';
			sync_inv1 <= '0';
			sync_inv2 <= '0';
			sync_inv3 <= '0';
			sync_inv4 <= '0';
			sync_inv5 <= '0';
			sync_ng1  <= '0';
			sync_ng2  <= '0';
			sync_ng3  <= '0';
			sync_ng4  <= '0';
			sync_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys = '1' and sync_det_cnt = X"00") then

				if ( (
					cnt_sync_lt( 6) = '1' or
					cnt_sync_lt( 7) = '1' or
					cnt_sync_lt( 8) = '1' or
					cnt_sync_lt( 9) = '1' or
					cnt_sync_lt( 0) = '1' ) and (
					cnt_sync_lt(22) = '0' and
					cnt_sync_lt(23) = '0' and
					cnt_sync_lt(24) = '0' and
					cnt_sync_lt(25) = '0' )
				) then
					sync_tmg1 <= '1';
				else
					sync_tmg1 <= '0';
				end if;

				if ( (
					cnt_sync_lt( 7) = '1' or
					cnt_sync_lt( 8) = '1' or
					cnt_sync_lt( 9) = '1' or
					cnt_sync_lt(10) = '1' or
					cnt_sync_lt(11) = '1' ) and (
					cnt_sync_lt(23) = '0' and
					cnt_sync_lt(24) = '0' and
					cnt_sync_lt(25) = '0' and
					cnt_sync_lt(26) = '0' )
				) then
					sync_tmg2 <= '1';
				else
					sync_tmg2 <= '0';
				end if;

				if ( (
					cnt_sync_lt( 8) = '1' or
					cnt_sync_lt( 9) = '1' or
					cnt_sync_lt(10) = '1' or
					cnt_sync_lt(11) = '1' or
					cnt_sync_lt(12) = '1' ) and (
					cnt_sync_lt(24) = '0' and
					cnt_sync_lt(25) = '0' and
					cnt_sync_lt(26) = '0' and
					cnt_sync_lt(27) = '0' )
				) then
					sync_tmg3 <= '1';
				else
					sync_tmg3 <= '0';
				end if;

				if ( (
					cnt_sync_lt( 9) = '1' or
					cnt_sync_lt(10) = '1' or
					cnt_sync_lt(11) = '1' or
					cnt_sync_lt(12) = '1' or
					cnt_sync_lt(13) = '1' ) and (
					cnt_sync_lt(25) = '0' and
					cnt_sync_lt(26) = '0' and
					cnt_sync_lt(27) = '0' and
					cnt_sync_lt(28) = '0' )
				) then
					sync_tmg4 <= '1';
				else
					sync_tmg4 <= '0';
				end if;

				if ( (
					cnt_sync_lt(10) = '1' or
					cnt_sync_lt(11) = '1' or
					cnt_sync_lt(12) = '1' or
					cnt_sync_lt(13) = '1' or
					cnt_sync_lt(14) = '1' ) and (
					cnt_sync_lt(26) = '0' and
					cnt_sync_lt(27) = '0' and
					cnt_sync_lt(28) = '0' and
					cnt_sync_lt(29) = '0' )
				) then
					sync_tmg5 <= '1';
				else
					sync_tmg5 <= '0';
				end if;

				if ( (
					cnt_sync_lt(21) = '1' or
					cnt_sync_lt(22) = '1' or
					cnt_sync_lt(23) = '1' or
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' ) and (
					cnt_sync_lt( 7) = '0' and
					cnt_sync_lt( 8) = '0' and
					cnt_sync_lt( 9) = '0' and
					cnt_sync_lt(10) = '0' )
				) then
					sync_inv1 <= '1';
				else
					sync_inv1 <= '0';
				end if;

				if ( (
					cnt_sync_lt(22) = '1' or
					cnt_sync_lt(23) = '1' or
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' ) and (
					cnt_sync_lt( 8) = '0' and
					cnt_sync_lt( 9) = '0' and
					cnt_sync_lt(10) = '0' and
					cnt_sync_lt(11) = '0' )
				) then
					sync_inv2 <= '1';
				else
					sync_inv2 <= '0';
				end if;

				if ( (
					cnt_sync_lt(23) = '1' or
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1' ) and (
					cnt_sync_lt( 9) = '0' and
					cnt_sync_lt(10) = '0' and
					cnt_sync_lt(11) = '0' and
					cnt_sync_lt(12) = '0' )
				) then
					sync_inv3 <= '1';
				else
					sync_inv3 <= '0';
				end if;

				if ( (
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1' or
					cnt_sync_lt(28) = '1' ) and (
					cnt_sync_lt(10) = '0' and
					cnt_sync_lt(11) = '0' and
					cnt_sync_lt(12) = '0' and
					cnt_sync_lt(13) = '0' )
				) then
					sync_inv4 <= '1';
				else
					sync_inv4 <= '0';
				end if;

				if ( (
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1' or
					cnt_sync_lt(28) = '1' or
					cnt_sync_lt(29) = '1' ) and (
					cnt_sync_lt(11) = '0' and
					cnt_sync_lt(12) = '0' and
					cnt_sync_lt(13) = '0' and
					cnt_sync_lt(14) = '0' )
				) then
					sync_inv5 <= '1';
				else
					sync_inv5 <= '0';
				end if;

				if (
					cnt_sync_lt(22) = '1' or
					cnt_sync_lt(23) = '1' or
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1'
				) then
					sync_ng1 <= '1';
				else
					sync_ng1 <= '0';
				end if;

				if (
					cnt_sync_lt(23) = '1' or
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1'
				) then
					sync_ng2 <= '1';
				else
					sync_ng2 <= '0';
				end if;

				if (
					cnt_sync_lt(24) = '1' or
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1'
				) then
					sync_ng3 <= '1';
				else
					sync_ng3 <= '0';
				end if;

				if (
					cnt_sync_lt(25) = '1' or
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1' or
					cnt_sync_lt(28) = '1'
				) then
					sync_ng4 <= '1';
				else
					sync_ng4 <= '0';
				end if;

				if (
					cnt_sync_lt(26) = '1' or
					cnt_sync_lt(27) = '1' or
					cnt_sync_lt(28) = '1' or
					cnt_sync_lt(29) = '1'
				) then
					sync_ng5 <= '1';
				else
					sync_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	sync_ng <= sync_ng1 and sync_ng2 and sync_ng3 and sync_ng4 and sync_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			clkinv    <= '0';
			clkinvcnt <= "00";
		elsif (CLK'event and CLK = '1') then
			if (det_ys = '1' and sync_det_cnt = X"01") then
				if (
					sync_tmg1 = '1' or sync_tmg2 = '1' or sync_tmg3 = '1' or
					sync_tmg4 = '1' or sync_tmg5 = '1'
				) then
					clkinvcnt <= "00";
				elsif (
					sync_inv1 = '1' or sync_inv2 = '1' or sync_inv3 = '1' or
					sync_inv4 = '1' or sync_inv5 = '1'
				) then
					if (clkinvcnt = "11") then
						clkinv <= not clkinv;
					else
						clkinvcnt <= clkinvcnt + 1;
					end if;
				end if;
			end if;
		end if;
	end process;

-- sync latch timing indicate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock3 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock3 <= '0';
			elsif (sync_ng = '1') then
				dt_lock3 <= '0';
			elsif (
				sync_tmg1 = '1' or
				sync_tmg2 = '1' or
				sync_tmg3 = '1' or
				sync_tmg4 = '1' or
				sync_tmg5 = '1'
			) then
				dt_lock3 <= '1';
			end if;
		end if;
	end process;


-- color data edge count
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cnt_yh    <= (others => '0');
			cnt_yh_lt <= (others => '0');
			cnt_yl    <= (others => '0');
			cnt_yl_lt <= (others => '0');
			cnt_bh    <= (others => '0');
			cnt_bh_lt <= (others => '0');
			cnt_bl    <= (others => '0');
			cnt_bl_lt <= (others => '0');
			cnt_rh    <= (others => '0');
			cnt_rh_lt <= (others => '0');
			cnt_rl    <= (others => '0');
			cnt_rl_lt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then
				cnt_yh    <= (others => '0');
				cnt_yh_lt <= cnt_yh;
				cnt_yl    <= (others => '0');
				cnt_yl_lt <= cnt_yl;
				cnt_bh    <= (others => '0');
				cnt_bh_lt <= cnt_bh;
				cnt_bl    <= (others => '0');
				cnt_bl_lt <= cnt_bl;
				cnt_rh    <= (others => '0');
				cnt_rh_lt <= cnt_rh;
				cnt_rl    <= (others => '0');
				cnt_rl_lt <= cnt_rl;
			else
				for i in 0 to 39 loop
					if (clkr_f(i+4) = '1') then
						cnt_yh(i) <= cnt_yh(i) or det_yh;
						cnt_yl(i) <= cnt_yl(i) or det_yl;
						cnt_bh(i) <= cnt_bh(i) or det_bh;
						cnt_bl(i) <= cnt_bl(i) or det_bl;
						cnt_rh(i) <= cnt_rh(i) or det_rh;
						cnt_rl(i) <= cnt_rl(i) or det_rl;
					end if;
				end loop;
			end if;
		end if;
	end process;


-- data latch timing generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			yh_tmg1 <= '0';
			yh_tmg2 <= '0';
			yh_tmg3 <= '0';
			yh_tmg4 <= '0';
			yh_tmg5 <= '0';
			yh_ng1  <= '0';
			yh_ng2  <= '0';
			yh_ng3  <= '0';
			yh_ng4  <= '0';
			yh_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_yh_lt(27) = '1' or
					cnt_yh_lt(28) = '1' or
					cnt_yh_lt(29) = '1' or
					cnt_yh_lt(30) = '1' or
					cnt_yh_lt(10) = '1' or
					cnt_yh_lt(11) = '1' or
					cnt_yh_lt(12) = '1' or
					cnt_yh_lt(13) = '1' ) and (
					cnt_yh_lt( 4) = '0' and
					cnt_yh_lt( 5) = '0' )
				) then
					yh_tmg1 <= '1';
				else
					yh_tmg1 <= '0';
				end if;

				if ( (
					cnt_yh_lt(28) = '1' or
					cnt_yh_lt(29) = '1' or
					cnt_yh_lt(30) = '1' or
					cnt_yh_lt(31) = '1' or
					cnt_yh_lt(11) = '1' or
					cnt_yh_lt(12) = '1' or
					cnt_yh_lt(13) = '1' or
					cnt_yh_lt(14) = '1' ) and (
					cnt_yh_lt( 5) = '0' and
					cnt_yh_lt( 6) = '0' )
				) then
					yh_tmg2 <= '1';
				else
					yh_tmg2 <= '0';
				end if;

				if ( (
					cnt_yh_lt(27) = '1' or
					cnt_yh_lt(28) = '1' or
					cnt_yh_lt(29) = '1' or
					cnt_yh_lt(30) = '1' or
					cnt_yh_lt(12) = '1' or
					cnt_yh_lt(13) = '1' or
					cnt_yh_lt(14) = '1' or
					cnt_yh_lt(15) = '1' ) and (
					cnt_yh_lt( 6) = '0' and
					cnt_yh_lt( 7) = '0' )
				) then
					yh_tmg3 <= '1';
				else
					yh_tmg3 <= '0';
				end if;

				if ( (
					cnt_yh_lt(28) = '1' or
					cnt_yh_lt(29) = '1' or
					cnt_yh_lt(30) = '1' or
					cnt_yh_lt(31) = '1' or
					cnt_yh_lt(13) = '1' or
					cnt_yh_lt(14) = '1' or
					cnt_yh_lt(15) = '1' or
					cnt_yh_lt(16) = '1' ) and (
					cnt_yh_lt( 7) = '0' and
					cnt_yh_lt( 8) = '0' )
				) then
					yh_tmg4 <= '1';
				else
					yh_tmg4 <= '0';
				end if;

				if ( (
					cnt_yh_lt(29) = '1' or
					cnt_yh_lt(30) = '1' or
					cnt_yh_lt(31) = '1' or
					cnt_yh_lt(32) = '1' or
					cnt_yh_lt(14) = '1' or
					cnt_yh_lt(15) = '1' or
					cnt_yh_lt(16) = '1' or
					cnt_yh_lt(17) = '1' ) and (
					cnt_yh_lt( 8) = '0' and
					cnt_yh_lt( 9) = '0' )
				) then
					yh_tmg5 <= '1';
				else
					yh_tmg5 <= '0';
				end if;

				if (
					cnt_yh_lt( 4) = '1' or
					cnt_yh_lt( 5) = '1'
				) then
					yh_ng1 <= '1';
				else
					yh_ng1 <= '0';
				end if;

				if (
					cnt_yh_lt( 5) = '1' or
					cnt_yh_lt( 6) = '1'
				) then
					yh_ng2 <= '1';
				else
					yh_ng2 <= '0';
				end if;

				if (
					cnt_yh_lt( 6) = '1' or
					cnt_yh_lt( 7) = '1'
				) then
					yh_ng3 <= '1';
				else
					yh_ng3 <= '0';
				end if;

				if (
					cnt_yh_lt( 7) = '1' or
					cnt_yh_lt( 8) = '1'
				) then
					yh_ng4 <= '1';
				else
					yh_ng4 <= '0';
				end if;

				if (
					cnt_yh_lt( 8) = '1' or
					cnt_yh_lt( 9) = '1'
				) then
					yh_ng5 <= '1';
				else
					yh_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	yh_ng <= yh_ng1 and yh_ng2 and yh_ng3 and yh_ng4 and yh_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			yl_tmg1 <= '0';
			yl_tmg2 <= '0';
			yl_tmg3 <= '0';
			yl_tmg4 <= '0';
			yl_tmg5 <= '0';
			yl_ng1  <= '0';
			yl_ng2  <= '0';
			yl_ng3  <= '0';
			yl_ng4  <= '0';
			yl_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_yl_lt(27) = '1' or
					cnt_yl_lt(28) = '1' or
					cnt_yl_lt(29) = '1' or
					cnt_yl_lt(30) = '1' or
					cnt_yl_lt(10) = '1' or
					cnt_yl_lt(11) = '1' or
					cnt_yl_lt(12) = '1' or
					cnt_yl_lt(13) = '1' ) and (
					cnt_yl_lt( 4) = '0' and
					cnt_yl_lt( 5) = '0' )
				) then
					yl_tmg1 <= '1';
				else
					yl_tmg1 <= '0';
				end if;

				if ( (
					cnt_yl_lt(28) = '1' or
					cnt_yl_lt(29) = '1' or
					cnt_yl_lt(30) = '1' or
					cnt_yl_lt(31) = '1' or
					cnt_yl_lt(11) = '1' or
					cnt_yl_lt(12) = '1' or
					cnt_yl_lt(13) = '1' or
					cnt_yl_lt(14) = '1' ) and (
					cnt_yl_lt( 5) = '0' and
					cnt_yl_lt( 6) = '0' )
				) then
					yl_tmg2 <= '1';
				else
					yl_tmg2 <= '0';
				end if;

				if ( (
					cnt_yl_lt(27) = '1' or
					cnt_yl_lt(28) = '1' or
					cnt_yl_lt(29) = '1' or
					cnt_yl_lt(30) = '1' or
					cnt_yl_lt(12) = '1' or
					cnt_yl_lt(13) = '1' or
					cnt_yl_lt(14) = '1' or
					cnt_yl_lt(15) = '1' ) and (
					cnt_yl_lt( 6) = '0' and
					cnt_yl_lt( 7) = '0' )
				) then
					yl_tmg3 <= '1';
				else
					yl_tmg3 <= '0';
				end if;

				if ( (
					cnt_yl_lt(28) = '1' or
					cnt_yl_lt(29) = '1' or
					cnt_yl_lt(30) = '1' or
					cnt_yl_lt(31) = '1' or
					cnt_yl_lt(13) = '1' or
					cnt_yl_lt(14) = '1' or
					cnt_yl_lt(15) = '1' or
					cnt_yl_lt(16) = '1' ) and (
					cnt_yl_lt( 7) = '0' and
					cnt_yl_lt( 8) = '0' )
				) then
					yl_tmg4 <= '1';
				else
					yl_tmg4 <= '0';
				end if;

				if ( (
					cnt_yl_lt(29) = '1' or
					cnt_yl_lt(30) = '1' or
					cnt_yl_lt(31) = '1' or
					cnt_yl_lt(32) = '1' or
					cnt_yl_lt(14) = '1' or
					cnt_yl_lt(15) = '1' or
					cnt_yl_lt(16) = '1' or
					cnt_yl_lt(17) = '1' ) and (
					cnt_yl_lt( 8) = '0' and
					cnt_yl_lt( 9) = '0' )
				) then
					yl_tmg5 <= '1';
				else
					yl_tmg5 <= '0';
				end if;

				if (
					cnt_yl_lt( 4) = '1' or
					cnt_yl_lt( 5) = '1'
				) then
					yl_ng1 <= '1';
				else
					yl_ng1 <= '0';
				end if;

				if (
					cnt_yl_lt( 5) = '1' or
					cnt_yl_lt( 6) = '1'
				) then
					yl_ng2 <= '1';
				else
					yl_ng2 <= '0';
				end if;

				if (
					cnt_yl_lt( 6) = '1' or
					cnt_yl_lt( 7) = '1'
				) then
					yl_ng3 <= '1';
				else
					yl_ng3 <= '0';
				end if;

				if (
					cnt_yl_lt( 7) = '1' or
					cnt_yl_lt( 8) = '1'
				) then
					yl_ng4 <= '1';
				else
					yl_ng4 <= '0';
				end if;

				if (
					cnt_yl_lt( 8) = '1' or
					cnt_yl_lt( 9) = '1'
				) then
					yl_ng5 <= '1';
				else
					yl_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	yl_ng <= yl_ng1 and yl_ng2 and yl_ng3 and yl_ng4 and yl_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			bh_tmg1 <= '0';
			bh_tmg2 <= '0';
			bh_tmg3 <= '0';
			bh_tmg4 <= '0';
			bh_tmg5 <= '0';
			bh_ng1  <= '0';
			bh_ng2  <= '0';
			bh_ng3  <= '0';
			bh_ng4  <= '0';
			bh_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_bh_lt(21) = '1' or
					cnt_bh_lt(22) = '1' or
					cnt_bh_lt(23) = '1' or
					cnt_bh_lt(24) = '1' or
					cnt_bh_lt(25) = '1' ) and (
					cnt_bh_lt( 7) = '0' and
					cnt_bh_lt( 8) = '0' and
					cnt_bh_lt( 9) = '0' and
					cnt_bh_lt(10) = '0' )
				) then
					bh_tmg1 <= '1';
				else
					bh_tmg1 <= '0';
				end if;

				if ( (
					cnt_bh_lt(22) = '1' or
					cnt_bh_lt(23) = '1' or
					cnt_bh_lt(24) = '1' or
					cnt_bh_lt(25) = '1' or
					cnt_bh_lt(26) = '1' ) and (
					cnt_bh_lt( 8) = '0' and
					cnt_bh_lt( 9) = '0' and
					cnt_bh_lt(10) = '0' and
					cnt_bh_lt(11) = '0' )
				) then
					bh_tmg2 <= '1';
				else
					bh_tmg2 <= '0';
				end if;

				if ( (
					cnt_bh_lt(23) = '1' or
					cnt_bh_lt(24) = '1' or
					cnt_bh_lt(25) = '1' or
					cnt_bh_lt(26) = '1' or
					cnt_bh_lt(27) = '1' ) and (
					cnt_bh_lt( 9) = '0' and
					cnt_bh_lt(10) = '0' and
					cnt_bh_lt(11) = '0' and
					cnt_bh_lt(12) = '0' )
				) then
					bh_tmg3 <= '1';
				else
					bh_tmg3 <= '0';
				end if;

				if ( (
					cnt_bh_lt(24) = '1' or
					cnt_bh_lt(25) = '1' or
					cnt_bh_lt(26) = '1' or
					cnt_bh_lt(27) = '1' or
					cnt_bh_lt(28) = '1' ) and (
					cnt_bh_lt(10) = '0' and
					cnt_bh_lt(11) = '0' and
					cnt_bh_lt(12) = '0' and
					cnt_bh_lt(13) = '0' )
				) then
					bh_tmg4 <= '1';
				else
					bh_tmg4 <= '0';
				end if;

				if ( (
					cnt_bh_lt(25) = '1' or
					cnt_bh_lt(26) = '1' or
					cnt_bh_lt(27) = '1' or
					cnt_bh_lt(28) = '1' or
					cnt_bh_lt(29) = '1' ) and (
					cnt_bh_lt(11) = '0' and
					cnt_bh_lt(12) = '0' and
					cnt_bh_lt(13) = '0' and
					cnt_bh_lt(14) = '0' )
				) then
					bh_tmg5 <= '1';
				else
					bh_tmg5 <= '0';
				end if;

				if (
					cnt_bh_lt( 7) = '1' or
					cnt_bh_lt( 8) = '1' or
					cnt_bh_lt( 9) = '1' or
					cnt_bh_lt(10) = '1'
				) then
					bh_ng1 <= '1';
				else
					bh_ng1 <= '0';
				end if;

				if (
					cnt_bh_lt( 8) = '1' or
					cnt_bh_lt( 9) = '1' or
					cnt_bh_lt(10) = '1' or
					cnt_bh_lt(11) = '1'
				) then
					bh_ng2 <= '1';
				else
					bh_ng2 <= '0';
				end if;

				if (
					cnt_bh_lt( 9) = '1' or
					cnt_bh_lt(10) = '1' or
					cnt_bh_lt(11) = '1' or
					cnt_bh_lt(12) = '1'
				) then
					bh_ng3 <= '1';
				else
					bh_ng3 <= '0';
				end if;

				if (
					cnt_bh_lt(10) = '1' or
					cnt_bh_lt(11) = '1' or
					cnt_bh_lt(12) = '1' or
					cnt_bh_lt(13) = '1'
				) then
					bh_ng4 <= '1';
				else
					bh_ng4 <= '0';
				end if;

				if (
					cnt_bh_lt(11) = '1' or
					cnt_bh_lt(12) = '1' or
					cnt_bh_lt(13) = '1' or
					cnt_bh_lt(14) = '1'
				) then
					bh_ng5 <= '1';
				else
					bh_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	bh_ng <= bh_ng1 and bh_ng2 and bh_ng3 and bh_ng4 and bh_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			bl_tmg1 <= '0';
			bl_tmg2 <= '0';
			bl_tmg3 <= '0';
			bl_tmg4 <= '0';
			bl_tmg5 <= '0';
			bl_ng1  <= '0';
			bl_ng2  <= '0';
			bl_ng3  <= '0';
			bl_ng4  <= '0';
			bl_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_bl_lt(21) = '1' or
					cnt_bl_lt(22) = '1' or
					cnt_bl_lt(23) = '1' or
					cnt_bl_lt(24) = '1' or
					cnt_bl_lt(25) = '1' ) and (
					cnt_bl_lt( 7) = '0' and
					cnt_bl_lt( 8) = '0' and
					cnt_bl_lt( 9) = '0' and
					cnt_bl_lt(10) = '0' )
				) then
					bl_tmg1 <= '1';
				else
					bl_tmg1 <= '0';
				end if;

				if ( (
					cnt_bl_lt(22) = '1' or
					cnt_bl_lt(23) = '1' or
					cnt_bl_lt(24) = '1' or
					cnt_bl_lt(25) = '1' or
					cnt_bl_lt(26) = '1' ) and (
					cnt_bl_lt( 8) = '0' and
					cnt_bl_lt( 9) = '0' and
					cnt_bl_lt(10) = '0' and
					cnt_bl_lt(11) = '0' )
				) then
					bl_tmg2 <= '1';
				else
					bl_tmg2 <= '0';
				end if;

				if ( (
					cnt_bl_lt(23) = '1' or
					cnt_bl_lt(24) = '1' or
					cnt_bl_lt(25) = '1' or
					cnt_bl_lt(26) = '1' or
					cnt_bl_lt(27) = '1' ) and (
					cnt_bl_lt( 9) = '0' and
					cnt_bl_lt(10) = '0' and
					cnt_bl_lt(11) = '0' and
					cnt_bl_lt(12) = '0' )
				) then
					bl_tmg3 <= '1';
				else
					bl_tmg3 <= '0';
				end if;

				if ( (
					cnt_bl_lt(24) = '1' or
					cnt_bl_lt(25) = '1' or
					cnt_bl_lt(26) = '1' or
					cnt_bl_lt(27) = '1' or
					cnt_bl_lt(28) = '1' ) and (
					cnt_bl_lt(10) = '0' and
					cnt_bl_lt(11) = '0' and
					cnt_bl_lt(12) = '0' and
					cnt_bl_lt(13) = '0' )
				) then
					bl_tmg4 <= '1';
				else
					bl_tmg4 <= '0';
				end if;

				if ( (
					cnt_bl_lt(25) = '1' or
					cnt_bl_lt(26) = '1' or
					cnt_bl_lt(27) = '1' or
					cnt_bl_lt(28) = '1' or
					cnt_bl_lt(29) = '1' ) and (
					cnt_bl_lt(11) = '0' and
					cnt_bl_lt(12) = '0' and
					cnt_bl_lt(13) = '0' and
					cnt_bl_lt(14) = '0' )
				) then
					bl_tmg5 <= '1';
				else
					bl_tmg5 <= '0';
				end if;

				if (
					cnt_bl_lt( 7) = '1' or
					cnt_bl_lt( 8) = '1' or
					cnt_bl_lt( 9) = '1' or
					cnt_bl_lt(10) = '1'
				) then
					bl_ng1 <= '1';
				else
					bl_ng1 <= '0';
				end if;

				if (
					cnt_bl_lt( 8) = '1' or
					cnt_bl_lt( 9) = '1' or
					cnt_bl_lt(10) = '1' or
					cnt_bl_lt(11) = '1'
				) then
					bl_ng2 <= '1';
				else
					bl_ng2 <= '0';
				end if;

				if (
					cnt_bl_lt( 9) = '1' or
					cnt_bl_lt(10) = '1' or
					cnt_bl_lt(11) = '1' or
					cnt_bl_lt(12) = '1'
				) then
					bl_ng3 <= '1';
				else
					bl_ng3 <= '0';
				end if;

				if (
					cnt_bl_lt(10) = '1' or
					cnt_bl_lt(11) = '1' or
					cnt_bl_lt(12) = '1' or
					cnt_bl_lt(13) = '1'
				) then
					bl_ng4 <= '1';
				else
					bl_ng4 <= '0';
				end if;

				if (
					cnt_bl_lt(11) = '1' or
					cnt_bl_lt(12) = '1' or
					cnt_bl_lt(13) = '1' or
					cnt_bl_lt(14) = '1'
				) then
					bl_ng5 <= '1';
				else
					bl_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	bl_ng <= bl_ng1 and bl_ng2 and bl_ng3 and bl_ng4 and bl_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			rh_tmg1 <= '0';
			rh_tmg2 <= '0';
			rh_tmg3 <= '0';
			rh_tmg4 <= '0';
			rh_tmg5 <= '0';
			rh_ng1  <= '0';
			rh_ng2  <= '0';
			rh_ng3  <= '0';
			rh_ng4  <= '0';
			rh_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_rh_lt(21) = '1' or
					cnt_rh_lt(22) = '1' or
					cnt_rh_lt(23) = '1' or
					cnt_rh_lt(24) = '1' or
					cnt_rh_lt(25) = '1' ) and (
					cnt_rh_lt( 7) = '0' and
					cnt_rh_lt( 8) = '0' and
					cnt_rh_lt( 9) = '0' and
					cnt_rh_lt(10) = '0' )
				) then
					rh_tmg1 <= '1';
				else
					rh_tmg1 <= '0';
				end if;

				if ( (
					cnt_rh_lt(22) = '1' or
					cnt_rh_lt(23) = '1' or
					cnt_rh_lt(24) = '1' or
					cnt_rh_lt(25) = '1' or
					cnt_rh_lt(26) = '1' ) and (
					cnt_rh_lt( 8) = '0' and
					cnt_rh_lt( 9) = '0' and
					cnt_rh_lt(10) = '0' and
					cnt_rh_lt(11) = '0' )
				) then
					rh_tmg2 <= '1';
				else
					rh_tmg2 <= '0';
				end if;

				if ( (
					cnt_rh_lt(23) = '1' or
					cnt_rh_lt(24) = '1' or
					cnt_rh_lt(25) = '1' or
					cnt_rh_lt(26) = '1' or
					cnt_rh_lt(27) = '1' ) and (
					cnt_rh_lt( 9) = '0' and
					cnt_rh_lt(10) = '0' and
					cnt_rh_lt(11) = '0' and
					cnt_rh_lt(12) = '0' )
				) then
					rh_tmg3 <= '1';
				else
					rh_tmg3 <= '0';
				end if;

				if ( (
					cnt_rh_lt(24) = '1' or
					cnt_rh_lt(25) = '1' or
					cnt_rh_lt(26) = '1' or
					cnt_rh_lt(27) = '1' or
					cnt_rh_lt(28) = '1' ) and (
					cnt_rh_lt(10) = '0' and
					cnt_rh_lt(11) = '0' and
					cnt_rh_lt(12) = '0' and
					cnt_rh_lt(13) = '0' )
				) then
					rh_tmg4 <= '1';
				else
					rh_tmg4 <= '0';
				end if;

				if ( (
					cnt_rh_lt(25) = '1' or
					cnt_rh_lt(26) = '1' or
					cnt_rh_lt(27) = '1' or
					cnt_rh_lt(28) = '1' or
					cnt_rh_lt(29) = '1' ) and (
					cnt_rh_lt(11) = '0' and
					cnt_rh_lt(12) = '0' and
					cnt_rh_lt(13) = '0' and
					cnt_rh_lt(14) = '0' )
				) then
					rh_tmg5 <= '1';
				else
					rh_tmg5 <= '0';
				end if;

				if (
					cnt_rh_lt( 7) = '1' or
					cnt_rh_lt( 8) = '1' or
					cnt_rh_lt( 9) = '1' or
					cnt_rh_lt(10) = '1'
				) then
					rh_ng1 <= '1';
				else
					rh_ng1 <= '0';
				end if;

				if (
					cnt_rh_lt( 8) = '1' or
					cnt_rh_lt( 9) = '1' or
					cnt_rh_lt(10) = '1' or
					cnt_rh_lt(11) = '1'
				) then
					rh_ng2 <= '1';
				else
					rh_ng2 <= '0';
				end if;

				if (
					cnt_rh_lt( 9) = '1' or
					cnt_rh_lt(10) = '1' or
					cnt_rh_lt(11) = '1' or
					cnt_rh_lt(12) = '1'
				) then
					rh_ng3 <= '1';
				else
					rh_ng3 <= '0';
				end if;

				if (
					cnt_rh_lt(10) = '1' or
					cnt_rh_lt(11) = '1' or
					cnt_rh_lt(12) = '1' or
					cnt_rh_lt(13) = '1'
				) then
					rh_ng4 <= '1';
				else
					rh_ng4 <= '0';
				end if;

				if (
					cnt_rh_lt(11) = '1' or
					cnt_rh_lt(12) = '1' or
					cnt_rh_lt(13) = '1' or
					cnt_rh_lt(14) = '1'
				) then
					rh_ng5 <= '1';
				else
					rh_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	rh_ng <= rh_ng1 and rh_ng2 and rh_ng3 and rh_ng4 and rh_ng5;


	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			rl_tmg1 <= '0';
			rl_tmg2 <= '0';
			rl_tmg3 <= '0';
			rl_tmg4 <= '0';
			rl_tmg5 <= '0';
			rl_ng1  <= '0';
			rl_ng2  <= '0';
			rl_ng3  <= '0';
			rl_ng4  <= '0';
			rl_ng5  <= '0';
		elsif (CLK'event and CLK = '1') then
			if (det_ys_flag = '1' and clkr_f(1) = '1' and vcnt = 2) then

				if ( (
					cnt_rl_lt(21) = '1' or
					cnt_rl_lt(22) = '1' or
					cnt_rl_lt(23) = '1' or
					cnt_rl_lt(24) = '1' or
					cnt_rl_lt(25) = '1' ) and (
					cnt_rl_lt( 7) = '0' and
					cnt_rl_lt( 8) = '0' and
					cnt_rl_lt( 9) = '0' and
					cnt_rl_lt(10) = '0' )
				) then
					rl_tmg1 <= '1';
				else
					rl_tmg1 <= '0';
				end if;

				if ( (
					cnt_rl_lt(22) = '1' or
					cnt_rl_lt(23) = '1' or
					cnt_rl_lt(24) = '1' or
					cnt_rl_lt(25) = '1' or
					cnt_rl_lt(26) = '1' ) and (
					cnt_rl_lt( 8) = '0' and
					cnt_rl_lt( 9) = '0' and
					cnt_rl_lt(10) = '0' and
					cnt_rl_lt(11) = '0' )
				) then
					rl_tmg2 <= '1';
				else
					rl_tmg2 <= '0';
				end if;

				if ( (
					cnt_rl_lt(23) = '1' or
					cnt_rl_lt(24) = '1' or
					cnt_rl_lt(25) = '1' or
					cnt_rl_lt(26) = '1' or
					cnt_rl_lt(27) = '1' ) and (
					cnt_rl_lt( 9) = '0' and
					cnt_rl_lt(10) = '0' and
					cnt_rl_lt(11) = '0' and
					cnt_rl_lt(12) = '0' )
				) then
					rl_tmg3 <= '1';
				else
					rl_tmg3 <= '0';
				end if;

				if ( (
					cnt_rl_lt(24) = '1' or
					cnt_rl_lt(25) = '1' or
					cnt_rl_lt(26) = '1' or
					cnt_rl_lt(27) = '1' or
					cnt_rl_lt(28) = '1' ) and (
					cnt_rl_lt(10) = '0' and
					cnt_rl_lt(11) = '0' and
					cnt_rl_lt(12) = '0' and
					cnt_rl_lt(13) = '0' )
				) then
					rl_tmg4 <= '1';
				else
					rl_tmg4 <= '0';
				end if;

				if ( (
					cnt_rl_lt(25) = '1' or
					cnt_rl_lt(26) = '1' or
					cnt_rl_lt(27) = '1' or
					cnt_rl_lt(28) = '1' or
					cnt_rl_lt(29) = '1' ) and (
					cnt_rl_lt(11) = '0' and
					cnt_rl_lt(12) = '0' and
					cnt_rl_lt(13) = '0' and
					cnt_rl_lt(14) = '0' )
				) then
					rl_tmg5 <= '1';
				else
					rl_tmg5 <= '0';
				end if;

				if (
					cnt_rl_lt( 7) = '1' or
					cnt_rl_lt( 8) = '1' or
					cnt_rl_lt( 9) = '1' or
					cnt_rl_lt(10) = '1'
				) then
					rl_ng1 <= '1';
				else
					rl_ng1 <= '0';
				end if;

				if (
					cnt_rl_lt( 8) = '1' or
					cnt_rl_lt( 9) = '1' or
					cnt_rl_lt(10) = '1' or
					cnt_rl_lt(11) = '1'
				) then
					rl_ng2 <= '1';
				else
					rl_ng2 <= '0';
				end if;

				if (
					cnt_rl_lt( 9) = '1' or
					cnt_rl_lt(10) = '1' or
					cnt_rl_lt(11) = '1' or
					cnt_rl_lt(12) = '1'
				) then
					rl_ng3 <= '1';
				else
					rl_ng3 <= '0';
				end if;

				if (
					cnt_rl_lt(10) = '1' or
					cnt_rl_lt(11) = '1' or
					cnt_rl_lt(12) = '1' or
					cnt_rl_lt(13) = '1'
				) then
					rl_ng4 <= '1';
				else
					rl_ng4 <= '0';
				end if;

				if (
					cnt_rl_lt(11) = '1' or
					cnt_rl_lt(12) = '1' or
					cnt_rl_lt(13) = '1' or
					cnt_rl_lt(14) = '1'
				) then
					rl_ng5 <= '1';
				else
					rl_ng5 <= '0';
				end if;

			end if;
		end if;
	end process;

	rl_ng <= rl_ng1 and rl_ng2 and rl_ng3 and rl_ng4 and rl_ng5;


-- data latch timing indicate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock41 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock41 <= '0';
			elsif (yh_ng = '1') then
				dt_lock41 <= '0';
			elsif (
				yh_tmg1 = '1' or
				yh_tmg2 = '1' or
				yh_tmg3 = '1' or
				yh_tmg4 = '1' or
				yh_tmg5 = '1'
			) then
				dt_lock41 <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock42 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock42 <= '0';
			elsif (yl_ng = '1') then
				dt_lock42 <= '0';
			elsif (
				yl_tmg1 = '1' or
				yl_tmg2 = '1' or
				yl_tmg3 = '1' or
				yl_tmg4 = '1' or
				yl_tmg5 = '1'
			) then
				dt_lock42 <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock51 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock51 <= '0';
			elsif (bh_ng = '1') then
				dt_lock51 <= '0';
			elsif (
				bh_tmg1 = '1' or
				bh_tmg2 = '1' or
				bh_tmg3 = '1' or
				bh_tmg4 = '1' or
				bh_tmg5 = '1'
			) then
				dt_lock51 <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock52 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock52 <= '0';
			elsif (bl_ng = '1') then
				dt_lock52 <= '0';
			elsif (
				bl_tmg1 = '1' or
				bl_tmg2 = '1' or
				bl_tmg3 = '1' or
				bl_tmg4 = '1' or
				bl_tmg5 = '1'
			) then
				dt_lock52 <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock53 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock53 <= '0';
			elsif (rh_ng = '1') then
				dt_lock53 <= '0';
			elsif (
				rh_tmg1 = '1' or
				rh_tmg2 = '1' or
				rh_tmg3 = '1' or
				rh_tmg4 = '1' or
				rh_tmg5 = '1'
			) then
				dt_lock53 <= '1';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock54 <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkdet_i = '0') then
				dt_lock54 <= '0';
			elsif (rl_ng = '1') then
				dt_lock54 <= '0';
			elsif (
				rl_tmg1 = '1' or
				rl_tmg2 = '1' or
				rl_tmg3 = '1' or
				rl_tmg4 = '1' or
				rl_tmg5 = '1'
			) then
				dt_lock54 <= '1';
			end if;
		end if;
	end process;

	dt_lock4 <= dt_lock41 or dt_lock42;
	dt_lock5 <= dt_lock51 or dt_lock52 or dt_lock53 or dt_lock54;


	sync_tmg <=
		clkr_f(30) when (sync_tmg3 = '1') else
		clkr_f(31) when (sync_tmg4 = '1') else
		clkr_f(29) when (sync_tmg2 = '1') else
		clkr_f(28) when (sync_tmg1 = '1') else
		clkr_f(32) when (sync_tmg5 = '1') else
		clkr_f(30);

	yh_tmg <=
		clkr_f(10) or clkr_f(25) when (yh_tmg3 = '1') else
		clkr_f(11) or clkr_f(26) when (yh_tmg4 = '1') else
		clkr_f( 9) or clkr_f(24) when (yh_tmg2 = '1') else
		clkr_f( 8) or clkr_f(23) when (yh_tmg1 = '1') else
		clkr_f(12) or clkr_f(27) when (yh_tmg5 = '1') else
		clkr_f(10) or clkr_f(25);

	yl_tmg <=
		clkr_f(10) or clkr_f(25) when (yl_tmg3 = '1') else
		clkr_f(11) or clkr_f(26) when (yl_tmg4 = '1') else
		clkr_f( 9) or clkr_f(24) when (yl_tmg2 = '1') else
		clkr_f( 8) or clkr_f(23) when (yl_tmg1 = '1') else
		clkr_f(12) or clkr_f(27) when (yl_tmg5 = '1') else
		clkr_f(10) or clkr_f(25);

	bh_tmg <=
		clkr_f(15) when (bh_tmg3 = '1') else
		clkr_f(16) when (bh_tmg4 = '1') else
		clkr_f(14) when (bh_tmg2 = '1') else
		clkr_f(13) when (bh_tmg1 = '1') else
		clkr_f(17) when (bh_tmg5 = '1') else
		clkr_f(15);

	bl_tmg <=
		clkr_f(15) when (bl_tmg3 = '1') else
		clkr_f(16) when (bl_tmg4 = '1') else
		clkr_f(14) when (bl_tmg2 = '1') else
		clkr_f(13) when (bl_tmg1 = '1') else
		clkr_f(17) when (bl_tmg5 = '1') else
		clkr_f(15);

	rh_tmg <=
		clkr_f(15) when (rh_tmg3 = '1') else
		clkr_f(16) when (rh_tmg4 = '1') else
		clkr_f(14) when (rh_tmg2 = '1') else
		clkr_f(13) when (rh_tmg1 = '1') else
		clkr_f(17) when (rh_tmg5 = '1') else
		clkr_f(15);

	rl_tmg <=
		clkr_f(15) when (rl_tmg3 = '1') else
		clkr_f(16) when (rl_tmg4 = '1') else
		clkr_f(14) when (rl_tmg2 = '1') else
		clkr_f(13) when (rl_tmg1 = '1') else
		clkr_f(17) when (rl_tmg5 = '1') else
		clkr_f(15);



	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			sync_tmg_f1 <= '0';
			yh_tmg_f1   <= '0';
			yl_tmg_f1   <= '0';
			bh_tmg_f1   <= '0';
			bl_tmg_f1   <= '0';
			rh_tmg_f1   <= '0';
			rl_tmg_f1   <= '0';
			clkenb_f1   <= '0';
		elsif (CLK'event and CLK = '1') then
			sync_tmg_f1 <= sync_tmg;
			yh_tmg_f1   <= yh_tmg;
			yl_tmg_f1   <= yl_tmg;
			bh_tmg_f1   <= bh_tmg;
			bl_tmg_f1   <= bl_tmg;
			rh_tmg_f1   <= rh_tmg;
			rl_tmg_f1   <= rl_tmg;
			clkenb_f1   <= clkr_f( 6) or clkr_f(21);
		end if;
	end process;


-- data latch
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dt_hsyncn_f1 <= '1';
			dt_hsyncn_f2 <= '1';
			dt_hsyncn_f3 <= '1';
			dt_hsyncn_f4 <= '1';
			dt_hsyncn_f5 <= '1';
			dt_hsyncn_f6 <= '1';
			dt_rgbh_f1   <= "000000";
			dt_rgbh_f2   <= "000000";
			dt_rgbh_f3   <= "000000";
			dt_rgbh_f4   <= "000000";
			dt_rgbh_f5   <= "000000";
			dt_rgbh_f6   <= "000000";
		elsif (CLK'event and CLK = '1') then
			if (sync_tmg_f1 = '1') then
				dt_hsyncn_f1 <= not p6_hsyncn_f3;
			end if;
			if (yh_tmg_f1 = '1') then
				dt_rgbh_f1(5) <= p6_rgbh_f3(5);
			end if;
			if (yl_tmg_f1 = '1') then
				dt_rgbh_f1(4) <= p6_rgbh_f3(4);
			end if;
			if (bh_tmg_f1 = '1') then
				dt_rgbh_f1(3) <= p6_rgbh_f3(3);
			end if;
			if (bl_tmg_f1 = '1') then
				dt_rgbh_f1(2) <= p6_rgbh_f3(2);
			end if;
			if (rh_tmg_f1 = '1') then
				dt_rgbh_f1(1) <= p6_rgbh_f3(1);
			end if;
			if (rl_tmg_f1 = '1') then
				dt_rgbh_f1(0) <= p6_rgbh_f3(0);
			end if;
			if (clkenb_f1 = '1') then
				dt_hsyncn_f2 <= dt_hsyncn_f1;
				dt_hsyncn_f3 <= dt_hsyncn_f2;
				dt_hsyncn_f4 <= dt_hsyncn_f3;
				dt_hsyncn_f5 <= dt_hsyncn_f4;
				dt_hsyncn_f6 <= dt_hsyncn_f5;
				dt_rgbh_f2   <= dt_rgbh_f1;
				dt_rgbh_f3   <= dt_rgbh_f2;
				dt_rgbh_f4   <= dt_rgbh_f3;
				dt_rgbh_f5   <= dt_rgbh_f4;
				dt_rgbh_f6   <= dt_rgbh_f5;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			yhdt_val <= '0';
			yldt_val <= '0';
		elsif (CLK'event and CLK = '1') then
			if (clkenb_f1 = '1') then
				if (
					dt_rgbh_f1(3 downto 0) = "0000" or	-- green
					dt_rgbh_f1(3 downto 0) = "0011" or	-- orange
					dt_rgbh_f1(3 downto 0) = "0101"		-- buff
				) then
					yhdt_val <= '1';
				else
					yhdt_val <= '0';
				end if;
				if (dt_rgbh_f1(3 downto 0) = "0011") then	-- orange
					yldt_val <= '1';
				else
					yldt_val <= '0';
				end if;
			end if;
		end if;
	end process;

	DT_HSYNCN <= dt_hsyncn_f6;
	DT_VSYNCN <= dt_vsyncn_i;
	CLKENB    <= clkr_f(8) or clkr_f(15) or clkr_f(23) or clkr_f(30);

	LOCK1 <= clkdet_i;
	LOCK2 <= dt_lock2;
	LOCK3 <= dt_lock3;
	LOCK4 <= dt_lock4;
	LOCK5 <= dt_lock5;

----		                                   YYBBRR
----		                                   HLHLHL
	DT_RGBH   <=
		"0100" when (dt_rgbh_f6             = "110000") else	-- dark green		(SC1/4)
		"1100" when (dt_rgbh_f6(3 downto 0) =   "0000") else	-- green			(SC1/2/3/4)
		"1110" when (dt_rgbh_f6(3 downto 0) =   "0001") else	-- yellow
		"0001" when (dt_rgbh_f6             = "110011") else	-- dark orange		(SC1)
		"0010" when (dt_rgbh_f6             = "010011") else	-- orange			(SC2/3)
		"0011" when (dt_rgbh_f6             = "000011") else	-- light orange		(SC1)
		"1101" when (dt_rgbh_f6(3 downto 0) =   "0100") else	-- cyan
		"0000" when (dt_rgbh_f6             = "110101") else	-- black			(SC1/2/4)
		"1111" when (dt_rgbh_f6(3 downto 0) =   "0101") else	-- buff				(SC2/3)
		"1010" when (dt_rgbh_f6(3 downto 0) =   "0111") else	-- red
		"1001" when (dt_rgbh_f6(3 downto 0) =   "1101") else	-- blue
		"1011" when (dt_rgbh_f6(3 downto 0) =   "1111") else	-- magenta
		"0000";


end RTL;
