--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity VGAOUT640 is
	port (
		RAMRLAT		: in  std_logic_vector(15 downto 0);
		RAMRLATENB	: in  std_logic;
		P6MODE		: in  std_logic;
		SRMODE		: in  std_logic;
		SC4COLORON	: in  std_logic;
		SC4COLORMD	: in  std_logic_vector(1 downto 0);
		DT_LOCK1	: in  std_logic;
		DT_LOCK2	: in  std_logic;
		LOCK_PLL	: in  std_logic;
		CLK50M		: in  std_logic;
		RSTN		: in  std_logic;
		RAMRADOUT	: out std_logic_vector(14 downto 0);
		RAMRADENB	: out std_logic;
		VGA_R		: out std_logic_vector(3 downto 0);
		VGA_G		: out std_logic_vector(3 downto 0);
		VGA_B		: out std_logic_vector(3 downto 0);
		VGA_HS		: out std_logic;
		VGA_VS		: out std_logic
	);
end VGAOUT640;

architecture RTL of VGAOUT640 is

	signal init_f1		: std_logic;
	signal init_f2		: std_logic;
	signal init_f3		: std_logic;

	signal ramrenb		: std_logic;
	signal ramrvenb		: std_logic;
	signal ramrad		: std_logic_vector(16 downto 0);
	signal ramrlat_f0	: std_logic_vector(15 downto 0);
	signal ramrlat_f1	: std_logic_vector(15 downto 0);
	signal ramrlat_f2	: std_logic_vector(15 downto 0);
	signal ramrlatenb_f0: std_logic;
	signal ramrlatenb_f1: std_logic;
	signal ramrlatenb_f2: std_logic;
	signal ramrdt		: std_logic_vector(3 downto 0);
	signal ramvcnt		: std_logic_vector(2 downto 0);
	signal ramadd_f0	: std_logic_vector(14 downto 0);
	signal ramradenb_f0	: std_logic;

	signal hcntrdenb	: std_logic;
	signal hcntrd		: std_logic_vector(10 downto 0);
	signal hcntrdtmp	: std_logic_vector(10 downto 0);
	signal vcntrd		: std_logic_vector(9 downto 0);

	signal vga_vs_i		: std_logic;

	signal vga_hs_i		: std_logic;
	signal r_hvld_i		: std_logic;
	signal r_hvld_f1	: std_logic;
	signal r_hvld_f2	: std_logic;
	signal r_hvld2_f1	: std_logic;
	signal r_hvld2_f2	: std_logic;
	signal r_hvld2_f3	: std_logic;
	signal r_hvld2_f4	: std_logic;
	signal r_hvld2_f5	: std_logic;
	signal r_hvld2_f6	: std_logic;
	signal r_hvld2_f7	: std_logic;
	signal r_hvld2_f8	: std_logic;
	signal r_hvld3_f1	: std_logic;
	signal r_hvld3_f2	: std_logic;
	signal r_hvld3_f3	: std_logic;
	signal r_hvld3_f4	: std_logic;
	signal r_hvld3_f5	: std_logic;
	signal r_hvld3_f6	: std_logic;
	signal r_hvld2_sel	: std_logic;

	signal ramrdt_mux			: std_logic_vector(3 downto 0);
	signal ramrdt_mux_tmp		: std_logic_vector(3 downto 0);
	signal ramrdt_mux_tmp_f1	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_tmp_f2	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_tmp_f3	: std_logic_vector(3 downto 0);

	signal ramrdt_mux_f1	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f2	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f3	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f4	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f5	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f6	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f7	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_f8	: std_logic_vector(3 downto 0);
	signal ramrdt_mux_sel	: std_logic_vector(3 downto 0);

	signal col_b_f2		: std_logic;
	signal col_b_f3		: std_logic;
	signal col_b_f4		: std_logic;
	signal col_b_f5		: std_logic;
	signal col_b_f6		: std_logic;
	signal col_b_f7		: std_logic;
	signal col_b_f8		: std_logic;
	signal col_b_f9		: std_logic;
	signal col_b_f10	: std_logic;
	signal col_w_f2		: std_logic;
	signal col_w_f3		: std_logic;
	signal col_w_f4		: std_logic;
	signal col_w_f5		: std_logic;
	signal col_w_f6		: std_logic;
	signal col_w_f7		: std_logic;
	signal col_w_f8		: std_logic;
	signal col_w_f9		: std_logic;
	signal col_w_f10	: std_logic;

	signal bitpat		: std_logic_vector(4 downto 0);
	signal bitpat_f1	: std_logic_vector(4 downto 0);
	signal bitpatsel	: std_logic_vector(4 downto 0);
	signal allsc4		: std_logic;
	signal allsc4_f1	: std_logic;
	signal allsc4sel	: std_logic;
	signal set2p		: std_logic_vector(23 downto 0);
	signal set2p_f1		: std_logic_vector(23 downto 0);
	signal set2p_f2		: std_logic_vector(23 downto 0);
	signal set2r		: std_logic_vector(23 downto 0);
	signal set2r_f1		: std_logic_vector(23 downto 0);
	signal set2r_f2		: std_logic_vector(23 downto 0);
	signal set2g		: std_logic_vector(23 downto 0);
	signal set2g_f1		: std_logic_vector(23 downto 0);
	signal set2g_f2		: std_logic_vector(23 downto 0);
	signal set2b		: std_logic_vector(23 downto 0);
	signal set2b_f1		: std_logic_vector(23 downto 0);
	signal set2b_f2		: std_logic_vector(23 downto 0);

	signal c_dg			: std_logic_vector(23 downto 0);
	signal c_g			: std_logic_vector(23 downto 0);
	signal c_dp			: std_logic_vector(23 downto 0);
	signal c_p			: std_logic_vector(23 downto 0);
	signal c_db			: std_logic_vector(23 downto 0);
	signal c_b			: std_logic_vector(23 downto 0);
	signal c_dr			: std_logic_vector(23 downto 0);
	signal c_r			: std_logic_vector(23 downto 0);
	signal c_w			: std_logic_vector(23 downto 0);
	signal c_00			: std_logic_vector(23 downto 0);


	signal vgadat		: std_logic_vector(23 downto 0);
	signal vgadat_p6	: std_logic_vector(23 downto 0);
	signal vgadat_mk	: std_logic_vector(23 downto 0);

	signal vga_hs_f1	: std_logic;
	signal vga_hs_f2	: std_logic;
	signal vga_hs_f3	: std_logic;
	signal vga_hs_f4	: std_logic;
	signal vga_hs_f5	: std_logic;
	signal vga_hs_f6	: std_logic;
	signal vga_hs_f7	: std_logic;
	signal vga_hs_f8	: std_logic;
	signal vga_hs_f9	: std_logic;
	signal vga_hs_f10	: std_logic;
	signal vga_hs_f11	: std_logic;
	signal vga_hs_f12	: std_logic;
	signal vga_hs_sel	: std_logic;
	signal vga_vs_f1	: std_logic;
	signal vga_vs_f2	: std_logic;
	signal vga_vs_f3	: std_logic;
	signal vga_vs_f4	: std_logic;
	signal vga_vs_f5	: std_logic;
	signal vga_vs_f6	: std_logic;
	signal vga_vs_f7	: std_logic;
	signal vga_vs_f8	: std_logic;
	signal vga_vs_f9	: std_logic;
	signal vga_vs_f10	: std_logic;
	signal vga_vs_f11	: std_logic;
	signal vga_vs_f12	: std_logic;
	signal vga_vs_sel	: std_logic;

	signal vga_r_o		: std_logic_vector(7 downto 0);
	signal vga_g_o		: std_logic_vector(7 downto 0);
	signal vga_b_o		: std_logic_vector(7 downto 0);
	signal vga_hs_o		: std_logic;
	signal vga_vs_o		: std_logic;

begin

-- ݑi14.31818MHzjPC912NbNi63.6952usj
-- ǂݏoi50MHzjQC3200NbNi64.000ussj
-- ǂݏo̕

---- MAIN reset
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			init_f1 <= '1';
			init_f2 <= '1';
			init_f3 <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (LOCK_PLL = '1') then
				init_f1 <= '0';
				init_f2 <= init_f1;
				init_f3 <= init_f2;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramadd_f0 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrd(2 downto 0) = "000") then			-- ramradd 荞
				ramadd_f0 <= ramrad(16 downto 2);
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramradenb_f0 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrd(2 downto 0) = "000") then
				ramradenb_f0 <= '1';
			elsif (hcntrd(2 downto 0) = "011") then
				ramradenb_f0 <= '0';
			end if;
		end if;
	end process;

	RAMRADOUT <= ramadd_f0;
	RAMRADENB <= ramradenb_f0;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrlatenb_f0 <= '0';
			ramrlatenb_f1 <= '0';
			ramrlatenb_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			ramrlatenb_f0 <= RAMRLATENB;
			ramrlatenb_f1 <= ramrlatenb_f0;
			ramrlatenb_f2 <= ramrlatenb_f1;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrlat_f0 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (ramrlatenb_f1 = '1' and ramrlatenb_f2 = '0') then
				ramrlat_f0 <= RAMRLAT;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrlat_f1 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '0') then
			if (hcntrd(2 downto 0) = "100") then
				ramrlat_f1 <= ramrlat_f0;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrlat_f2 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrd(2 downto 0) = "000") then		-- ramradd 荞݂WNbN
				ramrlat_f2 <= ramrlat_f1;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrdt <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if    (hcntrd(2 downto 0) = "001") then
				ramrdt <= ramrlat_f2( 3 downto  0);
			elsif (hcntrd(2 downto 0) = "011") then
				ramrdt <= ramrlat_f2( 7 downto  4);
			elsif (hcntrd(2 downto 0) = "101") then
				ramrdt <= ramrlat_f2(11 downto  8);
			elsif (hcntrd(2 downto 0) = "111") then
				ramrdt <= ramrlat_f2(15 downto 12);
			end if;
		end if;
	end process;


---- CLK50M read
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			hcntrdenb <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			hcntrdenb <= not init_f3;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			hcntrd <= (others => '1');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdenb = '0') then
				hcntrd <= (others => '1');
			elsif (hcntrd = 800*2-1) then
				hcntrd <= (others => '0');
			else
				hcntrd <= hcntrd + 1;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			hcntrdtmp <= (others => '1');
		elsif (CLK50M'event and CLK50M = '1') then
			hcntrdtmp <= hcntrd - (96+48)*2 + 1 - 4;
		end if;
	end process;



	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			vcntrd  <= (others => '1');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdenb = '0') then
				vcntrd  <= (others => '1');
			elsif (hcntrd = 800*2-1) then
				if (vcntrd = 525-1) then
					vcntrd  <= (others => '0');
				else
					vcntrd <= vcntrd + 1;
				end if;
			end if;
		end if;
	end process;


-- horizontal
-- sync=96 / back porch=48 / pixel=640 / front porch=16
-- all=800
--
-- vartical
-- sync=2 / back porch=33 / pixel=480 (36+408+36) / front porch=10
-- all=525
--

	vga_hs_i <=	'0' when (0 <= hcntrd and hcntrd < 96*2) else '1';
	vga_vs_i <= '0' when (0 <= vcntrd and vcntrd <    2) else '1';

	r_hvld_i <=
		'1' when ( (P6MODE = '1' or SRMODE = '0') and
			(96+48)*2 <= hcntrd and hcntrd < (96+48+640)*2 and
			(2+33+40) <= vcntrd and vcntrd < (2+33+40+400) ) else

		'1' when ( (P6MODE = '0' and SRMODE = '1') and
			(96+48)*2 <= hcntrd and hcntrd < (96+48+640)*2 and
			(2+33+36) <= vcntrd and vcntrd < (2+33+36+408) ) else

		'0';

	ramrenb <=
		'1' when ( (P6MODE = '1' or SRMODE = '0') and
			(96+48)*2-16 <= hcntrd and hcntrd < (96+48+640)*2-16 and
			(2+33+40)    <= vcntrd and vcntrd < (2+33+40+400) )  else

		'1' when ( (P6MODE = '0' and SRMODE = '1') and
			(96+48)*2-16 <= hcntrd and hcntrd < (96+48+640)*2-16 and
			(2+33+36)    <= vcntrd and vcntrd < (2+33+36+408) )  else

		'0';

	ramrvenb <=
		'1' when ( (P6MODE = '1' or SRMODE = '0') and
			(2+33+40) <= vcntrd and vcntrd < (2+33+40+400) ) else

		'1' when ( (P6MODE = '0' and SRMODE = '1') and
			(2+33+36) <= vcntrd and vcntrd < (2+33+36+408) ) else

		'0';


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrad  <= (others => '0');
			ramvcnt <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdenb = '0') then
				ramrad  <= (others => '0');
				ramvcnt <= (others => '0');
			elsif (hcntrd = 800*2-1) then
				if (vcntrd = 525-1) then
					ramrad  <= (others => '0');
					ramvcnt <= (others => '0');
				elsif (ramrvenb = '1') then
					if (ramvcnt = "001") then
						ramvcnt <= (others => '0');
					else
						ramvcnt <= ramvcnt + 1;
						ramrad  <= ramrad - 640;
					end if;
				end if;
			elsif (ramrenb = '1' and hcntrd(0 downto 0) = "1") then
				ramrad <= ramrad + 1;
			end if;
		end if;
	end process;



	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			vga_hs_f1  <= '1';
			vga_hs_f2  <= '1';
			vga_hs_f3  <= '1';
			vga_hs_f4  <= '1';
			vga_hs_f5  <= '1';
			vga_hs_f6  <= '1';
			vga_hs_f7  <= '1';
			vga_hs_f8  <= '1';
			vga_hs_f9  <= '1';
			vga_hs_f10 <= '1';
			vga_hs_f11 <= '1';
			vga_hs_f12 <= '1';
			vga_vs_f1  <= '1';
			vga_vs_f2  <= '1';
			vga_vs_f3  <= '1';
			vga_vs_f4  <= '1';
			vga_vs_f5  <= '1';
			vga_vs_f6  <= '1';
			vga_vs_f7  <= '1';
			vga_vs_f8  <= '1';
			vga_vs_f9  <= '1';
			vga_vs_f10 <= '1';
			vga_vs_f11 <= '1';
			vga_vs_f12 <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdtmp(0 downto 0) = "1") then
				vga_hs_f1  <= vga_hs_i;
				vga_hs_f2  <= vga_hs_f1;
				vga_hs_f3  <= vga_hs_f2;
				vga_hs_f4  <= vga_hs_f3;
				vga_hs_f5  <= vga_hs_f4;
				vga_hs_f6  <= vga_hs_f5;
				vga_hs_f7  <= vga_hs_f6;
				vga_hs_f8  <= vga_hs_f7;
				vga_hs_f9  <= vga_hs_f8;
				vga_hs_f10 <= vga_hs_f9;
				vga_hs_f11 <= vga_hs_f10;
				vga_hs_f12 <= vga_hs_f11;
				vga_vs_f1  <= vga_vs_i;
				vga_vs_f2  <= vga_vs_f1;
				vga_vs_f3  <= vga_vs_f2;
				vga_vs_f4  <= vga_vs_f3;
				vga_vs_f5  <= vga_vs_f4;
				vga_vs_f6  <= vga_vs_f5;
				vga_vs_f7  <= vga_vs_f6;
				vga_vs_f8  <= vga_vs_f7;
				vga_vs_f9  <= vga_vs_f8;
				vga_vs_f10 <= vga_vs_f9;
				vga_vs_f11 <= vga_vs_f10;
				vga_vs_f12 <= vga_vs_f11;
			end if;
		end if;
	end process;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			r_hvld_f1  <= '0';
			r_hvld_f2  <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			r_hvld_f1  <= r_hvld_i;
			r_hvld_f2  <= r_hvld_f1;
		end if;
	end process;

	ramrdt_mux_tmp <= ramrdt when (r_hvld_f2 = '1') else (others => '0');

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrdt_mux_tmp_f1 <= (others => '0');
			ramrdt_mux_tmp_f2 <= (others => '0');
			ramrdt_mux_tmp_f3 <= (others => '0');
			ramrdt_mux        <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			ramrdt_mux_tmp_f1  <= ramrdt_mux_tmp;
			ramrdt_mux_tmp_f2  <= ramrdt_mux_tmp_f1;
			if (hcntrdtmp(0 downto 0) = "1") then
				ramrdt_mux_tmp_f3  <= ramrdt_mux_tmp_f2;
				ramrdt_mux <= ramrdt_mux_tmp_f3;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			ramrdt_mux_f1  <= (others => '0');
			ramrdt_mux_f2  <= (others => '0');
			ramrdt_mux_f3  <= (others => '0');
			ramrdt_mux_f4  <= (others => '0');
			ramrdt_mux_f5  <= (others => '0');
			ramrdt_mux_f6  <= (others => '0');
			ramrdt_mux_f7  <= (others => '0');
			ramrdt_mux_f8  <= (others => '0');
			col_b_f2       <= '0';
			col_b_f3       <= '0';
			col_b_f4       <= '0';
			col_b_f5       <= '0';
			col_b_f6       <= '0';
			col_b_f7       <= '0';
			col_b_f8       <= '0';
			col_b_f9       <= '0';
			col_b_f10      <= '0';
			col_w_f2       <= '0';
			col_w_f3       <= '0';
			col_w_f4       <= '0';
			col_w_f5       <= '0';
			col_w_f6       <= '0';
			col_w_f7       <= '0';
			col_w_f8       <= '0';
			col_w_f9       <= '0';
			col_w_f10      <= '0';
			r_hvld2_f1     <= '0';
			r_hvld2_f2     <= '0';
			r_hvld2_f3     <= '0';
			r_hvld2_f4     <= '0';
			r_hvld2_f5     <= '0';
			r_hvld2_f6     <= '0';
			r_hvld2_f7     <= '0';
			r_hvld2_f8     <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdtmp(0 downto 0) = "1") then

				ramrdt_mux_f1  <= ramrdt_mux;
				ramrdt_mux_f2  <= ramrdt_mux_f1;
				ramrdt_mux_f3  <= ramrdt_mux_f2;
				ramrdt_mux_f4  <= ramrdt_mux_f3;
				ramrdt_mux_f5  <= ramrdt_mux_f4;
				ramrdt_mux_f6  <= ramrdt_mux_f5;
				ramrdt_mux_f7  <= ramrdt_mux_f6;
				ramrdt_mux_f8  <= ramrdt_mux_f7;

				if (ramrdt_mux(2 downto 0) = "000" and ramrdt_mux_f1(2 downto 0) = "000") then
					col_b_f2 <= '1';
				else
					col_b_f2 <= '0';
				end if;

				if (ramrdt_mux = "1111" and ramrdt_mux_f1 = "1111") then
					col_w_f2 <= '1';
				else
					col_w_f2 <= '0';
				end if;

				col_b_f3       <= col_b_f2;
				col_b_f4       <= col_b_f3;
				col_b_f5       <= col_b_f4;
				col_b_f6       <= col_b_f5;
				col_b_f7       <= col_b_f6;
				col_b_f8       <= col_b_f7;
				col_b_f9       <= col_b_f8;
				col_b_f10      <= col_b_f9;
				col_w_f3       <= col_w_f2;
				col_w_f4       <= col_w_f3;
				col_w_f5       <= col_w_f4;
				col_w_f6       <= col_w_f5;
				col_w_f7       <= col_w_f6;
				col_w_f8       <= col_w_f7;
				col_w_f9       <= col_w_f8;
				col_w_f10      <= col_w_f9;

				r_hvld2_f1     <= r_hvld_f2;
				r_hvld2_f2     <= r_hvld2_f1;
				r_hvld2_f3     <= r_hvld2_f2;
				r_hvld2_f4     <= r_hvld2_f3;
				r_hvld2_f5     <= r_hvld2_f4;
				r_hvld2_f6     <= r_hvld2_f5;
				r_hvld2_f7     <= r_hvld2_f6;
				r_hvld2_f8     <= r_hvld2_f7;

			end if;
		end if;
	end process;


-- SC4COLORON    = 0/1 SCREEN4 color off/on
-- SC4COLORMD(0) = 0/1 color normal/reverse
-- SC4COLORMD(1) = 0/1 GREEN-PINK / RED-BLUE


--   Even
--   v
-- x000x 	@@@@@@
-- x001x 	@@@@@@
-- x010x 	@@@@@@
-- x011x 	@@@@@@
-- x100x 	×΁^@@@
-- 01010 	@@΁@@@@
-- 01011 	@΁^×΁@@
-- 11010 	@@΁@@@@
-- 11011 	@@΁@@@@
-- x110x 	@@@@@@
-- x111x 	@@@@@@
-- 43210

--   Odd
--   v
-- x000x 	@@@@@@
-- x001x 	@@@@@@
-- x010x 	@@΁@@@@
-- x011x 	@@@@@@
-- x100x 	Ó^@@@
-- 01010 	@@@@@@
-- 01011 	Ó^@@@
-- 11010 	@@@@@@
-- 11011 	@@@@@@
-- x110x 	@@@@@@
-- x111x 	@@@@@@
-- 43210


	bitpat <=
		col_w_f10 & col_w_f8 & col_w_f6 & col_w_f4 & col_w_f2;

	allsc4 <=
			'0' when (col_w_f10 = '0' and col_b_f10 = '0') else
			'0' when (col_w_f8  = '0' and col_b_f8  = '0') else
			'0' when (col_w_f6  = '0' and col_b_f6  = '0') else
			'0' when (col_w_f4  = '0' and col_b_f4  = '0') else
			'0' when (col_w_f2  = '0' and col_b_f2  = '0') else
			'1';


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			bitpat_f1 <= (others => '0');
			allsc4_f1 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdtmp(0 downto 0) = "1") then
				bitpat_f1 <= bitpat;
				allsc4_f1 <= allsc4;
			end if;
		end if;
	end process;

	bitpatsel <= bitpat when (hcntrdtmp(1) = '0') else bitpat_f1;
	allsc4sel <= allsc4 when (hcntrdtmp(1) = '0') else allsc4_f1;


	c_dg <= X"1E9764";	-- ɂ݈×
	c_g  <= X"26FF5C";	-- ɂݗ
	c_dp <= X"970069";	-- ɂ݈Ó
	c_p  <= X"FF00EC";	-- ɂݓ
	c_db <= X"1E5096";	-- ɂ݈Ð
	c_b  <= X"26C9FF";	-- ɂݐ
	c_dr <= X"962D00";	-- ɂ݈Ð
	c_r  <= X"FF3000";	-- ɂݐ
	c_w  <= X"E0FFD0";	-- ɂݔ
	c_00 <= X"000000";	-- 


	set2p <=
		c_00 when (allsc4sel = '0') else
		c_00 when (SC4COLORON = '0') else
		c_00 when (SC4COLORMD(1) = '0') else
		c_00 when (bitpatsel(3 downto 1) = "000") else

		c_00 when (bitpatsel(3 downto 1) = "001") else

		c_p  when (bitpatsel(3 downto 1) = "010") else
		c_w  when (bitpatsel(3 downto 1) = "011") else

		c_dg when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "00") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "01") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2) = '1') else

		c_g  when (bitpatsel            = "01010") else

		c_g  when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "00") else
		c_dg when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "01") else
		c_g  when (bitpatsel            = "01011" and hcntrdtmp(2) = '1') else

		c_g  when (bitpatsel(4 downto 1)= "1101") else
		c_w  when (bitpatsel(3 downto 1) = "110") else
		c_w  when (bitpatsel(3 downto 1) = "111") else
		c_00;

	set2g <=
		c_00 when (allsc4sel = '0') else
		c_00 when (SC4COLORON = '0') else
		c_00 when (SC4COLORMD(1) = '0') else
		c_00 when (bitpatsel(3 downto 1) = "000") else

		c_00 when (bitpatsel(3 downto 1) = "001") else

		c_g  when (bitpatsel(3 downto 1) = "010") else
		c_w  when (bitpatsel(3 downto 1) = "011") else

		c_dp when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "00") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "01") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2) = '1') else

		c_p  when (bitpatsel            = "01010") else

		c_p  when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "00") else
		c_dp when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "01") else
		c_p  when (bitpatsel            = "01011" and hcntrdtmp(2) = '1') else

		c_p  when (bitpatsel(4 downto 1)= "1101") else
		c_w  when (bitpatsel(3 downto 1) = "110") else
		c_w  when (bitpatsel(3 downto 1) = "111") else
		c_00;



	set2r <=
		c_00 when (allsc4sel = '0') else
		c_00 when (SC4COLORON = '0') else
		c_00 when (SC4COLORMD(1) = '1') else
		c_00 when (bitpatsel(3 downto 1) = "000") else

		c_00 when (bitpatsel(3 downto 1) = "001") else

		c_r  when (bitpatsel(3 downto 1) = "010") else
		c_w  when (bitpatsel(3 downto 1) = "011") else

		c_db when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "00") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "01") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2) = '1') else

		c_b  when (bitpatsel            = "01010") else

		c_b  when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "00") else
		c_db when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "01") else
		c_b  when (bitpatsel            = "01011" and hcntrdtmp(2) = '1') else

		c_b  when (bitpatsel(4 downto 1)= "1101") else
		c_w  when (bitpatsel(3 downto 1) = "110") else
		c_w  when (bitpatsel(3 downto 1) = "111") else
		c_00;

	set2b <=
		c_00 when (allsc4sel = '0') else
		c_00 when (SC4COLORON = '0') else
		c_00 when (SC4COLORMD(1) = '1') else
		c_00 when (bitpatsel(3 downto 1) = "000") else

		c_00 when (bitpatsel(3 downto 1) = "001") else

		c_b  when (bitpatsel(3 downto 1) = "010") else
		c_w  when (bitpatsel(3 downto 1) = "011") else

		c_dr when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "00") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2 downto 1) = "01") else
		c_00 when (bitpatsel(3 downto 1) = "100"  and hcntrdtmp(2) = '1') else

		c_r  when (bitpatsel            = "01010") else

		c_r  when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "00") else
		c_dr when (bitpatsel            = "01011" and hcntrdtmp(2 downto 1) = "01") else
		c_r  when (bitpatsel            = "01011" and hcntrdtmp(2) = '1') else

		c_r  when (bitpatsel(4 downto 1)= "1101") else
		c_w  when (bitpatsel(3 downto 1) = "110") else
		c_w  when (bitpatsel(3 downto 1) = "111") else
		c_00;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			set2p_f1 <= (others => '0');
			set2p_f2 <= (others => '0');
			set2r_f1 <= (others => '0');
			set2r_f2 <= (others => '0');
			set2g_f1 <= (others => '0');
			set2g_f2 <= (others => '0');
			set2b_f1 <= (others => '0');
			set2b_f2 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdtmp(0 downto 0) = "1") then
				set2p_f1 <= set2p;
				set2p_f2 <= set2p_f1;
				set2r_f1 <= set2r;
				set2r_f2 <= set2r_f1;
				set2g_f1 <= set2g;
				set2g_f2 <= set2g_f1;
				set2b_f1 <= set2b;
				set2b_f2 <= set2b_f1;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			r_hvld3_f1 <= '0';
			r_hvld3_f2 <= '0';
			r_hvld3_f3 <= '0';
			r_hvld3_f4 <= '0';
			r_hvld3_f5 <= '0';
			r_hvld3_f6 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			r_hvld3_f1 <= r_hvld2_f8;
			r_hvld3_f2 <= r_hvld3_f1;
			r_hvld3_f3 <= r_hvld3_f2;
			r_hvld3_f4 <= r_hvld3_f3;
			r_hvld3_f5 <= r_hvld3_f4;
			r_hvld3_f6 <= r_hvld3_f5;
		end if;
	end process;

	r_hvld2_sel    <= r_hvld3_f6;
	ramrdt_mux_sel <= ramrdt_mux_f8;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			vgadat_p6 <= (others => '0');
			vgadat_mk <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (hcntrdtmp(0 downto 0) = "1") then
				case ramrdt_mux_f7 is
					when "0100" => vgadat_p6 <= X"083600";	-- #1
					when "1100" => vgadat_p6 <= X"17A100";	-- #2
--					when "0101" => vgadat_p6 <= X"24FF00";	-- #3		(no use)
					when "1110" => vgadat_p6 <= X"D4FF00";	-- #3
					when "0001" => vgadat_p6 <= X"361400";	-- #1
					when "0010" => vgadat_p6 <= X"A13D00";	-- #2
					when "0011" => vgadat_p6 <= X"FF6100";	-- #3
					when "1101" => vgadat_p6 <= X"00FFA6";	-- #3
					when "0000" => vgadat_p6 <= X"050505";	-- #1ij
--					when "1000" => vgadat_p6 <= X"A1A1A1;	-- #2		(no use)
					when "1111" => vgadat_p6 <= X"FFFFFF";	-- #3
					when "1010" => vgadat_p6 <= X"FF004D";	-- #3
					when "1001" => vgadat_p6 <= X"1F00FF";	-- #3
					when "1011" => vgadat_p6 <= X"DF00FF";	-- #3
					when others => vgadat_p6 <= X"000000";
				end case;

				case ramrdt_mux_f7 is
					when "0000" => vgadat_mk <= X"000000";	-- mk2 ()
					when "0001" => vgadat_mk <= X"B000FF";	-- mk2 
					when "0010" => vgadat_mk <= X"FFB000";	-- mk2 
					when "0011" => vgadat_mk <= X"FF00B0";	-- mk2 Ԏ
					when "0100" => vgadat_mk <= X"00FFB0";	-- mk2 
					when "0101" => vgadat_mk <= X"00B0FF";	-- mk2 F
					when "0110" => vgadat_mk <= X"B0FF00";	-- mk2 
					when "0111" => vgadat_mk <= X"B0B0B0";	-- mk2 DF
					when "1000" => vgadat_mk <= X"000000";	-- mk2 
					when "1001" => vgadat_mk <= X"0000FF";	-- mk2 
					when "1010" => vgadat_mk <= X"FF0000";	-- mk2 
					when "1011" => vgadat_mk <= X"FF00FF";	-- mk2 }[^
					when "1100" => vgadat_mk <= X"00FF00";	-- mk2 
					when "1101" => vgadat_mk <= X"00FFFF";	-- mk2 VA
					when "1110" => vgadat_mk <= X"FFFF00";	-- mk2 
					when "1111" => vgadat_mk <= X"FFFFFF";	-- mk2 
					when others => vgadat_mk <= X"000000";
				end case;

			end if;
		end if;
	end process;


	vgadat <=
		X"000000" when (r_hvld2_sel = '0') else

		X"1E5096" when (DT_LOCK1 = '0' or DT_LOCK2 = '0') else		-- no sync

		set2p_f2  when (set2p_f2 /= X"000000" and hcntrdtmp(2) =     SC4COLORMD(0) ) else
		set2r_f2  when (set2r_f2 /= X"000000" and hcntrdtmp(2) =     SC4COLORMD(0) ) else
		set2g_f2  when (set2g_f2 /= X"000000" and hcntrdtmp(2) = not SC4COLORMD(0) ) else
		set2b_f2  when (set2b_f2 /= X"000000" and hcntrdtmp(2) = not SC4COLORMD(0) ) else

		vgadat_p6 when (P6MODE = '1') else
		vgadat_mk;


	vga_hs_sel <= vga_hs_f12;
	vga_vs_sel <= vga_vs_f12;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			vga_r_o  <= (others => '0');
			vga_g_o  <= (others => '0');
			vga_b_o  <= (others => '0');
			vga_hs_o <= '1';
			vga_vs_o <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			vga_r_o  <= vgadat(23 downto 16);
			vga_g_o  <= vgadat(15 downto  8);
			vga_b_o  <= vgadat( 7 downto  0);
			vga_hs_o <= vga_hs_sel;
			vga_vs_o <= vga_vs_sel;
		end if;
	end process;

	VGA_R	<= vga_r_o(7 downto 4);
	VGA_G	<= vga_g_o(7 downto 4);
	VGA_B	<= vga_b_o(7 downto 4);
	VGA_HS	<= vga_hs_o;
	VGA_VS	<= vga_vs_o;

end RTL;
